@@ -1,35 +1,50 @@
-from babel import Locale
 from django.conf import settings
+from django.core.exceptions import ImproperlyConfigured
 from django.forms import Select, TextInput
 from django.forms.widgets import MultiWidget
 from django.utils import translation
 from phonenumbers import PhoneNumberFormat
 from phonenumbers.data import _COUNTRY_CODE_TO_REGION_CODE
-from phonenumbers.phonenumberutil import region_code_for_number
+from phonenumbers.phonenumberutil import (
+    national_significant_number,
+    region_code_for_number,
+)
 
 from phonenumber_field.phonenumber import PhoneNumber
 
+try:
+    import babel
+except ModuleNotFoundError:
+    babel = None
+
 
 class PhonePrefixSelect(Select):
     initial = None
 
     def __init__(self, initial=None):
+        if babel is None:
+            raise ImproperlyConfigured(
+                "The PhonePrefixSelect widget requires the babel package be installed."
+            )
+
         choices = [("", "---------")]
         language = translation.get_language() or settings.LANGUAGE_CODE
-        if language:
-            locale = Locale(translation.to_locale(language))
-            for prefix, values in _COUNTRY_CODE_TO_REGION_CODE.items():
-                prefix = "+%d" % prefix
-                if initial and initial in values:
-                    self.initial = prefix
-                for country_code in values:
-                    country_name = locale.territories.get(country_code)
-                    if country_name:
-                        choices.append((prefix, "{} {}".format(country_name, prefix)))
+        locale = babel.Locale(translation.to_locale(language))
+        if not initial:
+            region = getattr(settings, "PHONENUMBER_DEFAULT_REGION", None)
+            initial = region
+        for prefix, values in _COUNTRY_CODE_TO_REGION_CODE.items():
+            prefix = "+%d" % prefix
+            if initial and initial in values:
+                self.initial = prefix
+            for country_code in values:
+                country_name = locale.territories.get(country_code)
+                if country_name:
+                    choices.append((prefix, "{} {}".format(country_name, prefix)))
         super().__init__(choices=sorted(choices, key=lambda item: item[1]))
 
-    def render(self, name, value, *args, **kwargs):
-        return super().render(name, value or self.initial, *args, **kwargs)
+    def get_context(self, name, value, attrs):
+        return super().get_context(name, value or self.initial, attrs)
 
 
 class PhoneNumberPrefixWidget(MultiWidget):
@@ -47,7 +62,10 @@ class PhoneNumberPrefixWidget(MultiWidget):
         if value:
             if type(value) == PhoneNumber:
                 if value.country_code and value.national_number:
-                    return ["+%d" % value.country_code, value.national_number]
+                    return [
+                        "+%d" % value.country_code,
+                        national_significant_number(value),
+                    ]
             else:
                 return value.split(".")
         return [None, ""]
