@@ -9,11 +9,12 @@ from phonenumbers import phonenumberutil
 from phonenumber_field import formfields, modelfields
 from phonenumber_field.modelfields import PhoneNumberField
 from phonenumber_field.phonenumber import PhoneNumber, to_python
-from phonenumber_field.widgets import PhoneNumberInternationalFallbackWidget
 
 from . import models
 from .forms import CustomPhoneNumberFormField, PhoneNumberForm
 
+ALGERIAN_PHONE_NUMBER = "+213799136332"
+
 
 def phone_transform(obj):
     return (obj.pk, obj.name, obj.phone)
@@ -30,6 +31,28 @@ class PhoneNumberFieldTestCase(TestCase):
     }
     invalid_numbers = ["+44 113 892111"]
 
+    def test_str_for_valid_number(self):
+        self.assertEqual(
+            str(PhoneNumber.from_string(self.test_number_1)), self.test_number_1,
+        )
+
+    def test_str_for_invalid_number(self):
+        self.assertEqual(
+            str(to_python("invalid")), "invalid",
+        )
+
+    def test_repr_for_invalid_number(self):
+        self.assertEqual(
+            repr(to_python("invalid")), "InvalidPhoneNumber(raw_input=invalid)",
+        )
+
+    def test_repr_for_valid_number(self):
+        self.assertTrue(
+            repr(PhoneNumber.from_string(self.test_number_1)).startswith(
+                "PhoneNumber("
+            ),
+        )
+
     def test_valid_numbers_are_valid(self):
         numbers = [
             PhoneNumber.from_string(number_string)
@@ -49,6 +72,9 @@ class PhoneNumberFieldTestCase(TestCase):
         ]
         self.assertTrue(all(not number.is_valid() for number in numbers))
 
+    def test_same_invalid_numbers_are_equal(self):
+        self.assertEqual(PhoneNumber(raw_input="invalid"), "invalid")
+
     def test_objects_with_same_number_are_equal(self):
         numbers = [
             models.MandatoryPhoneNumber.objects.create(
@@ -182,26 +208,6 @@ class PhoneNumberFieldTestCase(TestCase):
                     self.storage_numbers[frmt][1],
                 )
 
-    def test_fallback_widget_switches_between_national_and_international(self):
-        region, number_string = self.local_numbers[0]
-        number = PhoneNumber.from_string(number_string, region=region)
-        gb_widget = PhoneNumberInternationalFallbackWidget(region="GB")
-        de_widget = PhoneNumberInternationalFallbackWidget(region="DE")
-        self.assertHTMLEqual(
-            gb_widget.render("number", number),
-            '<input name="number" type="text" value="01606 75178" />',
-        )
-        self.assertHTMLEqual(
-            de_widget.render("number", number),
-            '<input name="number" type="text" value="+44 1606 75178" />',
-        )
-
-        # If there's been a validation error, the value should be included verbatim
-        self.assertHTMLEqual(
-            gb_widget.render("number", "error"),
-            '<input name="number" type="text" value="error" />',
-        )
-
     def test_phone_number_form_empty_value(self):
         form = PhoneNumberForm({"phone_number": ""})
 
@@ -213,22 +219,43 @@ class PhoneNumberFieldTestCase(TestCase):
         m = models.MandatoryPhoneNumber.objects.defer("phone_number").get(pk=m.pk)
         self.assertEqual(m.phone_number, self.test_number_1)
 
-    def test_filter_invalid(self):
-        models.NullablePhoneNumber.objects.create(phone_number=self.test_number_1)
-        models.NullablePhoneNumber.objects.create(phone_number="")
-        models.NullablePhoneNumber.objects.create(phone_number=None)
-        with self.assertRaises(ValueError) as cm:
-            models.NullablePhoneNumber.objects.filter(phone_number="123").exists()
-        self.assertEqual(force_text(cm.exception), "“123” is not a valid phone number.")
+    def test_filtering_by_invalid_number_does_not_raise_value_error(self):
+        models.NullablePhoneNumber.objects.filter(phone_number="123")
 
-    def test_filter_int(self):
-        models.NullablePhoneNumber.objects.create(phone_number=self.test_number_1)
-        models.NullablePhoneNumber.objects.create(phone_number="")
-        models.NullablePhoneNumber.objects.create(phone_number=None)
+    def test_filter_by_invalid_type_int_raises_value_error(self):
         msg = "Can't convert int to PhoneNumber"
         with self.assertRaisesMessage(TypeError, msg):
             models.NullablePhoneNumber.objects.filter(phone_number=123).exists()
 
+    def test_filter_by_invalid_number_does_not_match_empty_or_none(self):
+        models.NullablePhoneNumber.objects.create(phone_number="")
+        models.NullablePhoneNumber.objects.create(phone_number=None)
+        self.assertFalse(
+            models.NullablePhoneNumber.objects.filter(phone_number="123").exists()
+        )
+
+    def test_filter_by_invalid_number_does_not_match_partial(self):
+        models.NullablePhoneNumber.objects.create(phone_number="+4930123456")
+        self.assertFalse(
+            models.NullablePhoneNumber.objects.filter(phone_number="123").exists()
+        )
+
+    def test_filter_by_partial_number_of_valid_number_works(self):
+        models.NullablePhoneNumber.objects.create(phone_number="+4923399999")
+        models.NullablePhoneNumber.objects.create(phone_number="+4930123456")
+        self.assertEqual(
+            models.NullablePhoneNumber.objects.filter(
+                phone_number__contains="9999"
+            ).count(),
+            1,
+        )
+        self.assertEqual(
+            models.NullablePhoneNumber.objects.filter(
+                phone_number__contains="123"
+            ).count(),
+            1,
+        )
+
 
 class PhonenumerFieldAppTest(TestCase):
     def test_save_field_to_database(self):
@@ -247,37 +274,32 @@ class PhonenumerFieldAppTest(TestCase):
             transform=phone_transform,
         )
 
-    def test_create_invalid_number(self):
-        with self.assertRaises(ValueError) as cm:
-            models.TestModel.objects.create(phone="invalid")
-        self.assertEqual(
-            force_text(cm.exception), "“invalid” is not a valid phone number."
-        )
+    def test_create_with_invalid_number_works(self):
+        obj = models.TestModel.objects.create(phone="invalid")
+        self.assertEqual(obj.phone.raw_input, "invalid")
+        obj = models.TestModel.objects.get(id=obj.id)
+        self.assertEqual(obj.phone.raw_input, "invalid")
 
-    def test_save_invalid_number(self):
+    def test_save_invalid_number_works(self):
         tm = models.TestModel.objects.create(phone="+1 604-333-4444")
         tm.phone = "invalid"
-        with self.assertRaises(ValueError) as cm:
-            tm.save()
-        self.assertEqual(
-            force_text(cm.exception), "“invalid” is not a valid phone number."
-        )
+        tm.save()
+        tm.refresh_from_db()
+        self.assertEqual(tm.phone.raw_input, "invalid")
 
-    def test_save_update_field_invalid_number(self):
+    def test_save_change_to_invalid_number_works(self):
         tm = models.TestModel.objects.create(phone="+1 604-333-4444")
         tm.phone = "invalid"
-        with self.assertRaises(ValueError) as cm:
-            tm.save(update_fields=["phone"])
-        self.assertEqual(
-            force_text(cm.exception), "“invalid” is not a valid phone number."
-        )
+        tm.save(update_fields=["phone"])
+        self.assertEqual(tm.phone.raw_input, "invalid")
+        tm.refresh_from_db()
+        self.assertEqual(tm.phone.raw_input, "invalid")
 
-    def test_update_to_invalid_number(self):
-        with self.assertRaises(ValueError) as cm:
-            models.TestModel.objects.update(phone="invalid")
-        self.assertEqual(
-            force_text(cm.exception), "“invalid” is not a valid phone number."
-        )
+    def test_queryset_update_to_invalid_number_works(self):
+        tm = models.TestModel.objects.create(phone="+1 604-333-4444")
+        models.TestModel.objects.filter(id=tm.id).update(phone="invalid")
+        tm.refresh_from_db()
+        self.assertEqual(tm.phone.raw_input, "invalid")
 
     def test_save_blank_phone_to_database(self):
         """Field Test for when Blank"""
@@ -368,39 +390,6 @@ class PhonenumerFieldAppTest(TestCase):
         self.assertIsInstance(models.TestModel.phone, modelfields.PhoneNumberDescriptor)
 
 
-class PhoneNumberFormFieldTest(TestCase):
-    def test_error_message(self):
-        class PhoneNumberForm(forms.Form):
-            number = formfields.PhoneNumberField()
-
-        form = PhoneNumberForm({"number": "invalid"})
-        self.assertIs(form.is_valid(), False)
-        self.assertEqual(
-            form.errors, {"number": ["Enter a valid phone number (e.g. +12125552368)."]}
-        )
-
-    def test_override_error_message(self):
-        class MyPhoneNumberField(formfields.PhoneNumberField):
-            default_error_messages = {"invalid": "MY INVALID MESSAGE!"}
-
-        class PhoneNumberForm(forms.Form):
-            number = MyPhoneNumberField()
-
-        form = PhoneNumberForm({"number": "invalid"})
-        self.assertIs(form.is_valid(), False)
-        self.assertEqual(form.errors, {"number": ["MY INVALID MESSAGE!"]})
-
-    def test_override_error_message_inline(self):
-        class PhoneNumberForm(forms.Form):
-            number = formfields.PhoneNumberField(
-                error_messages={"invalid": "MY INLINE INVALID MESSAGE!"}
-            )
-
-        form = PhoneNumberForm({"number": "invalid"})
-        self.assertIs(form.is_valid(), False)
-        self.assertEqual(form.errors, {"number": ["MY INLINE INVALID MESSAGE!"]})
-
-
 class RegionPhoneNumberFormFieldTest(TestCase):
     def test_regional_phone(self):
         class PhoneNumberForm(forms.Form):
@@ -446,7 +435,7 @@ class RegionPhoneNumberModelFieldTest(TestCase):
         self.assertEqual(phonenumbers.parse("+33612345678"), m.cell_number)
 
     def test_accepts_international_numbers(self):
-        num = "+16041234567"
+        num = "+16042234567"
         m = models.FrenchPhoneOwner(cell_number=num)
         self.assertEqual(phonenumbers.parse(num), m.cell_number)
 
@@ -520,3 +509,10 @@ class RegionPhoneNumberModelFieldTest(TestCase):
     def test_region_none(self):
         field = modelfields.PhoneNumberField()
         self.assertIsNone(field.region)
+
+    def test_algerian_phone_number_in_model(self):
+        m = models.MandatoryPhoneNumber.objects.create(
+            phone_number=ALGERIAN_PHONE_NUMBER
+        )
+        self.assertEqual(phonenumbers.parse(ALGERIAN_PHONE_NUMBER), m.phone_number)
+        self.assertEqual(ALGERIAN_PHONE_NUMBER, m.phone_number)
